@@ -3,7 +3,8 @@ package HTTP::Tiny;
 use strict;
 use warnings;
 # ABSTRACT: A small, simple, correct HTTP/1.1 client
-our $VERSION = '0.048'; # VERSION
+
+our $VERSION = '0.052';
 
 use Carp ();
 
@@ -53,9 +54,9 @@ use Carp ();
 #pod
 #pod The C<keep_alive> parameter enables a persistent connection, but only to a
 #pod single destination scheme, host and port.  Also, if any connection-relevant
-#pod attributes are modified, a persistent connection will be dropped.  If you want
-#pod persistent connections across multiple destinations, use multiple HTTP::Tiny
-#pod objects.
+#pod attributes are modified, or if the process ID or thread ID change, the
+#pod persistent connection will be dropped.  If you want persistent connections
+#pod across multiple destinations, use multiple HTTP::Tiny objects.
 #pod
 #pod See L</SSL SUPPORT> for more on the C<verify_SSL> and C<SSL_options> attributes.
 #pod
@@ -140,7 +141,9 @@ sub _set_proxies {
 
     # http proxy
     if (! exists $self->{http_proxy} ) {
-        $self->{http_proxy} = $ENV{http_proxy} || $self->{proxy};
+        # under CGI, bypass HTTP_PROXY as request sets it from Proxy header
+        local $ENV{HTTP_PROXY} if $ENV{REQUEST_METHOD};
+        $self->{http_proxy} = $ENV{http_proxy} || $ENV{HTTP_PROXY} || $self->{proxy};
     }
 
     if ( defined $self->{http_proxy} ) {
@@ -471,7 +474,7 @@ my %DefaultPort = (
 sub _agent {
     my $class = ref($_[0]) || $_[0];
     (my $default_agent = $class) =~ s{::}{-}g;
-    return $default_agent . "/" . ($class->VERSION || 0);
+    return $default_agent . "/" . $class->VERSION;
 }
 
 sub _request {
@@ -483,6 +486,7 @@ sub _request {
         method    => $method,
         scheme    => $scheme,
         host      => $host,
+        port      => $port,
         host_port => ($port == $DefaultPort{$scheme} ? $host : "$host:$port"),
         uri       => $path_query,
         headers   => {},
@@ -619,9 +623,9 @@ sub _create_proxy_tunnel {
 
     my $connect_request = {
         method    => 'CONNECT',
-        uri       => $request->{host_port},
+        uri       => "$request->{host}:$request->{port}",
         headers   => {
-            host => $request->{host_port},
+            host => "$request->{host}:$request->{port}",
             'user-agent' => $agent,
         }
     };
@@ -924,6 +928,8 @@ sub connect {
     $self->{scheme} = $scheme;
     $self->{host} = $host;
     $self->{port} = $port;
+    $self->{pid} = $$;
+    $self->{tid} = _get_tid();
 
     return $self;
 }
@@ -1367,7 +1373,9 @@ sub _assert_ssl {
 sub can_reuse {
     my ($self,$scheme,$host,$port) = @_;
     return 0 if
-         length($self->{rbuf})
+        $self->{pid} != $$
+        || $self->{tid} != _get_tid()
+        || length($self->{rbuf})
         || $scheme ne $self->{scheme}
         || $host ne $self->{host}
         || $port ne $self->{port}
@@ -1400,6 +1408,12 @@ sub _find_CA_file {
       . qq/Try installing Mozilla::CA from CPAN\n/;
 }
 
+# for thread safety, we need to know thread id if threads are loaded
+sub _get_tid {
+    no warnings 'reserved'; # for 'threads'
+    return threads->can("tid") ? threads->tid : 0;
+}
+
 sub _ssl_args {
     my ($self, $host) = @_;
 
@@ -1444,7 +1458,7 @@ HTTP::Tiny - A small, simple, correct HTTP/1.1 client
 
 =head1 VERSION
 
-version 0.048
+version 0.052
 
 =head1 SYNOPSIS
 
@@ -1554,9 +1568,9 @@ content field in the response will contain the text of the exception.
 
 The C<keep_alive> parameter enables a persistent connection, but only to a
 single destination scheme, host and port.  Also, if any connection-relevant
-attributes are modified, a persistent connection will be dropped.  If you want
-persistent connections across multiple destinations, use multiple HTTP::Tiny
-objects.
+attributes are modified, or if the process ID or thread ID change, the
+persistent connection will be dropped.  If you want persistent connections
+across multiple destinations, use multiple HTTP::Tiny objects.
 
 See L</SSL SUPPORT> for more on the C<verify_SSL> and C<SSL_options> attributes.
 
@@ -1830,7 +1844,7 @@ HTTP::Tiny supports the following proxy environment variables:
 
 =item *
 
-http_proxy
+http_proxy or HTTP_PROXY
 
 =item *
 
@@ -1842,6 +1856,11 @@ all_proxy or ALL_PROXY
 
 =back
 
+If the C<REQUEST_METHOD> environment variable is set, then this might be a CGI
+process and C<HTTP_PROXY> would be set from the C<Proxy:> header, which is a
+security risk.  If C<REQUEST_METHOD> is set, C<HTTP_PROXY> (the upper case
+variant only) is ignored.
+
 Tunnelling C<https> over an C<http> proxy using the CONNECT method is
 supported.  If your proxy uses C<https> itself, you can not tunnel C<https>
 over it.
@@ -2009,7 +2028,7 @@ David Golden <dagolden@cpan.org>
 
 =head1 CONTRIBUTORS
 
-=for stopwords Alan Gardner James Raspass Jess Robinson Lukas Eklund Martin J. Evans Martin-Louis Bright Mike Doherty Petr Písař Serguei Trouchelle Syohei YOSHIDA Sören Kornetzki Alessandro Ghedini Tom Hukins Tony Cook Brad Gilbert Chris Nehren Weyl Claes Jakobsson Clinton Gormley Craig Berry David Mitchell Edward Zborowski
+=for stopwords Alan Gardner Alessandro Ghedini Brad Gilbert Chris Nehren Weyl Claes Jakobsson Clinton Gormley Craig Berry David Mitchell Dean Pearce Edward Zborowski James Raspass Jess Robinson Lukas Eklund Martin J. Evans Martin-Louis Bright Mike Doherty Petr Písař Serguei Trouchelle Sören Kornetzki Syohei YOSHIDA Tom Hukins Tony Cook
 
 =over 4
 
@@ -2019,6 +2038,46 @@ Alan Gardner <gardner@pythian.com>
 
 =item *
 
+Alessandro Ghedini <al3xbio@gmail.com>
+
+=item *
+
+Brad Gilbert <bgills@cpan.org>
+
+=item *
+
+Chris Nehren <apeiron@cpan.org>
+
+=item *
+
+Chris Weyl <cweyl@alumni.drew.edu>
+
+=item *
+
+Claes Jakobsson <claes@surfar.nu>
+
+=item *
+
+Clinton Gormley <clint@traveljury.com>
+
+=item *
+
+Craig Berry <cberry@cpan.org>
+
+=item *
+
+David Mitchell <davem@iabyn.com>
+
+=item *
+
+Dean Pearce <pearce@pythian.com>
+
+=item *
+
+Edward Zborowski <ed@rubensteintech.com>
+
+=item *
+
 James Raspass <jraspass@gmail.com>
 
 =item *
@@ -2051,15 +2110,11 @@ Serguei Trouchelle <stro@cpan.org>
 
 =item *
 
-Syohei YOSHIDA <syohex@gmail.com>
-
-=item *
-
 Sören Kornetzki <soeren.kornetzki@delti.com>
 
 =item *
 
-Alessandro Ghedini <al3xbio@gmail.com>
+Syohei YOSHIDA <syohex@gmail.com>
 
 =item *
 
@@ -2069,38 +2124,6 @@ Tom Hukins <tom@eborcom.com>
 
 Tony Cook <tony@develop-help.com>
 
-=item *
-
-Brad Gilbert <bgills@cpan.org>
-
-=item *
-
-Chris Nehren <apeiron@cpan.org>
-
-=item *
-
-Chris Weyl <rsrchboy@cpan.org>
-
-=item *
-
-Claes Jakobsson <claes@surfar.nu>
-
-=item *
-
-Clinton Gormley <clint@traveljury.com>
-
-=item *
-
-Craig Berry <cberry@cpan.org>
-
-=item *
-
-David Mitchell <davem@iabyn.com>
-
-=item *
-
-Edward Zborowski <ed@rubensteintech.com>
-
 =back
 
 =head1 COPYRIGHT AND LICENSE
